Package: geeVerse
Type: Package
Title: A Comprehensive Analysis of High Dimensional Longitudinal Data
Version: 0.3.1
Authors@R: c(
    person("Tianhai", "Zu", email = "zuti@mail.uc.edu", role = c("aut", "cre")),
    person("Brittany", "Green", email = "brittany.green.1@louisville.edu", role = c("aut","ctb")),
    person("Yan", "Yu", email = "yan.yu@uc.edu", role = c("aut","ctb"))
  )
Description: To provide a comprehensive analysis of high dimensional longitudinal
    data,this package provides analysis for any combination of 1) simultaneous
    variable selection and estimation, 2) mean regression or quantile regression
    for heterogeneous data, 3) cross-sectional or longitudinal data, 4) balanced
    or imbalanced data, 5) moderate, high or even ultra-high dimensional data,
    via computationally efficient implementations of penalized generalized
    estimating equations.
License: GPL-3
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.5.0)
Imports: mvtnorm, quantreg, Rcpp (>= 0.10.2), doParallel, foreach, MASS
Suggests: methods
LinkingTo: Rcpp, RcppEigen
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2025-10-13 16:06:31 UTC; tianhaizu
Author: Tianhai Zu [aut, cre],
  Brittany Green [aut, ctb],
  Yan Yu [aut, ctb]
Maintainer: Tianhai Zu <zuti@mail.uc.edu>
Repository: CRAN
Date/Publication: 2025-10-13 16:20:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 07:35:34 UTC; unix
Archs: geeVerse.so.dSYM
