# Generated by roxygen2: do not edit by hand

S3method(coef,qpgee)
S3method(compile_result,default)
S3method(compile_result,qpgee)
S3method(predict,qpgee)
S3method(print,qpgee)
S3method(print,summary.qpgee)
S3method(qpgee,default)
S3method(qpgee,formula)
S3method(summary,qpgee)
export(CVfit)
export(PGEE)
export(Siga_cov)
export(compile_result)
export(geeVerse)
export(generate_data)
export(qpgee)
export(qpgeeControl)
import(Rcpp)
importFrom(MASS,ginv)
importFrom(doParallel,registerDoParallel)
importFrom(foreach,"%do%")
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(parallel,detectCores)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(stats,coef)
importFrom(stats,delete.response)
importFrom(stats,dnorm)
importFrom(stats,gaussian)
importFrom(stats,model.extract)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,na.omit)
importFrom(stats,pnorm)
importFrom(stats,sd)
importFrom(stats,terms)
useDynLib(geeVerse)
